import java.io.*;
import javax.media.*;
import javax.media.datasink.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.util.*;
import javax.media.control.*;
import java.util.*;
import java.awt.*;

//import com.sun.media.vfw.VFWCapture;         // JMF 2.1.1c version
import com.sun.media.protocol.vfw.VFWCapture;   // JMF 2.1.1e version

public class WebcamManager {

    protected Player 				player;
    protected CaptureDeviceInfo 	webCamDeviceInfo;
    protected MediaLocator 			ml;
    protected FormatControl 		formatControl;
    protected Component 			visualComponent;

    protected VideoFormat 			currentFormat;
    protected Format[] 				videoFormats;



    public WebcamManager() {
        super();
	}


    // AutoDetect the first web camera in the system
    // searches for video for windows (vfw) capture devices
    // returns the device info object if found, null otherwise
    private CaptureDeviceInfo autoDetect() {
        Vector list = CaptureDeviceManager.getDeviceList(null);

        CaptureDeviceInfo devInfo = null;
        //System.out.println(list);
        if (list != null) {
            String name;
            for (int i=0; i<list.size(); i++) {
                devInfo = (CaptureDeviceInfo)list.elementAt(i);
                name = devInfo.getName();

                if (name.startsWith("vfw:")) break;
            }

            if (devInfo != null && devInfo.getName().startsWith("vfw:"))
                return devInfo;
            else {
                for (int i=0; i<10; i++) {
                    try {
                        name = VFWCapture.capGetDriverDescriptionName(i);
                        if (name != null && name.length() > 1) {
                            devInfo = com.sun.media.protocol.vfw.VFWSourceStream.autoDetect(i);
                            if (devInfo != null) {
                                return devInfo;
                            }
                        }
                    }
                    catch (Exception ioEx) {
                        // ignore errors detecting device
                        //statusBar.setText("AutoDetect failed : " + ioEx.getMessage());
                    }
                }
                return null;
            }
        }
        else
            return null;
    }



 	public void initialize() throws WebcamProblemException {
 		// Try finding a webcam

 		webCamDeviceInfo = autoDetect();
 		if (webCamDeviceInfo == null)
 			throw new WebcamProblemException("Cannot detect webcam");

 		ml = webCamDeviceInfo.getLocator();
 		if (ml == null)
 			throw new WebcamProblemException("No MediaLocator for " + webCamDeviceInfo.getName());

 		try {
 			player = Manager.createRealizedPlayer(ml);
 		}
 		catch (Exception e) {
 			throw new WebcamProblemException("Cannot create player for " + webCamDeviceInfo.getName());
 		}
        if (player == null)
        	throw new WebcamProblemException("Cannot create player for " + webCamDeviceInfo.getName());

        player.start();
        formatControl = (FormatControl)player.getControl("javax.media.control.FormatControl");
        videoFormats = webCamDeviceInfo.getFormats();

        visualComponent = player.getVisualComponent();
        if (visualComponent == null)
        	throw new WebcamProblemException("Cannot get visual component for " + webCamDeviceInfo.getName());


 		Format currFormat = formatControl.getFormat();
        if (currFormat instanceof VideoFormat)
        	currentFormat = (VideoFormat)currFormat;
        else
        	throw new WebcamProblemException("Cannot get current video format for " + webCamDeviceInfo.getName());

        for (int i=0; i<videoFormats.length; i++) {
        	VideoFormat	f = (VideoFormat)videoFormats[i];
        	if ((f.getSize().getWidth() == RobotTracker.CAMERA_WIDTH) && (f.getSize().getHeight() == RobotTracker.CAMERA_HEIGHT) && (f.getMaxDataLength() == (RobotTracker.CAMERA_WIDTH*RobotTracker.CAMERA_HEIGHT*3))) {
        		setFormat(f);
        		break;
        	}
        }
 	}


    public CaptureDeviceInfo  getWebCamDeviceInfo() { return webCamDeviceInfo; }
	public MediaLocator 	  getMediaLocator() { return ml; }
	public Player 			  getPlayer() { return player; }
	public Format[] 	  	  getVideoFormats() { return videoFormats; }
	public VideoFormat 		  getCurrentFormat() { return currentFormat; }





    // Set the format to the desired selected one
    public void setFormat(VideoFormat selectedFormat) throws WebcamProblemException {
        if (formatControl == null)
        	throw new WebcamProblemException("Visual component not an instance of FormatControl");

        player.stop();
        formatControl.setFormat(selectedFormat);
        player.start();

        currentFormat = selectedFormat;
    }


    /*// DeviceInfo - outputs text information
    public void deviceInfo() {
        if (webCamDeviceInfo != null) {
            Format[] formats = webCamDeviceInfo.getFormats();

            if ((formats != null) && (formats.length > 0)) {
            }

            for (int i=0; i<formats.length; i++) {
                Format aFormat = formats[i];
                if (aFormat instanceof VideoFormat) {
                    Dimension dim = ((VideoFormat)aFormat).getSize();
                    // System.out.println("Video Format " + i + " : " + formats[i].getEncoding() + ", " + dim.width + " x " + dim.height);
                }
            }
        }
        else {
            System.out.println("Error : No web cam detected");
        }
    }*/

    // Grabs and returns a frame's buffer, as an image, from the web cam / device
    public Image grabFrameImage() throws WebcamProblemException {
        Buffer buffer = grabFrameBuffer();
        if (buffer == null)
        	throw new WebcamProblemException("Buffer grabbed is null");

        // Convert it to an image
        BufferToImage btoi = new BufferToImage((VideoFormat)buffer.getFormat());
        if (btoi == null)
      		throw new WebcamProblemException("Cannot create BufferToImage instance");

        Image image = btoi.createImage(buffer);
        if (image == null)
			throw new WebcamProblemException("BufferToImage cannot convert buffer");

        return image;
    }


    // Grabs and returns a frame's buffer from the web cam / device
    private Buffer grabFrameBuffer() throws WebcamProblemException {
        if (player == null)
        	throw new WebcamProblemException("Player is null");

        FrameGrabbingControl fgc = (FrameGrabbingControl)player.getControl("javax.media.control.FrameGrabbingControl");
        if (fgc == null)
        	throw new WebcamProblemException("FrameGrabbingControl is null");

        return fgc.grabFrame();
    }


    // Closes and cleans up the player
    public void closePlayer() {
        if (player != null) {
            player.close();
            player.deallocate();
            player = null;
        }
    }
}

